unit Game;

interface

uses Crt, Defs, Graphics, Palette, Tex, Procs, Monsters, Player, Plats;

type
    PGame = ^TGame;
    TGame = object
    private
      Tab, AllowTab, Demo: Boolean;
      BegQ, EndQ: PMonster;
      Screen: Pointer;
      LevelInfo: Info;
      Screen_X, Screen_Y: Integer;
      Mario: PMario;
      SpawnX, SpawnY: Integer;
      CurrentLevel, Gold: Integer;
      Speeds: array[1..6] of Byte;
    public
      constructor Init;
      destructor Done;
      procedure Run;
      procedure LoadLevel(LevNum: Integer);
      procedure LoadData;
      procedure Intro;
      procedure ShowLevelNum;
      procedure DrawInfo;
      procedure RefreshScreen;
      procedure GameOver;
      procedure Sizes;
      procedure WinGame;
    end;

implementation

constructor TGame.Init;
  var i: Integer;
begin
  BegQ := nil;
  EndQ := nil;
  LoadData;
  LoadFont(14, FONT_FILE);
  SetMode($13);
  Screen := SetBuf;
  SetScreenBuf(Screen);
  SetNullPal;
  SetTexPal;
  if not FindParam(SKIP_PARAM) then Intro;
  AllowTab := FindParam(TAB_PARAM);
  Demo := FindParam(DEMO_PARAM);
  Mario := New(PMario, Init);
  GetMem(Level, SizeOf(TLevel));
  CurrentLevel := 0;
  LoadLevel(0);
  for i := 1 to 6 do
    Speeds[i] := SPEED_TABLE[i];
  Mario^.Lives := MAX_LIVES;
  Tab := False;
end;

destructor TGame.Done;
begin
  FreeMem(Level, SizeOf(TLevel));
  if BegQ <> nil then Dispose(BegQ, Done);

  FreeBuf(Screen);
  FreeMem(Data, DATA_SIZE);
  SetMode(3);
  Write('Mario v', VERSION:1:2);
  if UNREGISTERED then
    Writeln(' - ', MESSAGE)
  else
    Writeln;
  Writeln(COPYRIGHT);
end;

procedure DrawBack;
  var i: Integer;
      C: Byte;
begin
  SetTexPal;
  for i := 202 to 218 do begin
    C := (i - 202) * 15;
    SetRGBPal(i, 0, 0, C);
    C := 255 - (i - 202) * 15;
    SetRGBPal(i + 16, 0, 0, C);
  end;
  Texture(0, 0, 0);
end;

procedure TGame.ShowLevelNum;
  var S: String;
      Count: Byte;
begin
  Str(CurrentLevel + 1, S);
  DrawBack;
  OutString(131, 91, 'World ' + S, 0);
  OutString(130, 90, 'World ' + S, 250);
  PutImage(150, 60, 20, 14, 0, 76 * 14, 20, Data);
  CopyToScreen(Screen);
  FadeIn;
  Count := 0;
  repeat
    Inc(Count);
    VertRet;
  until Count = 100;
  FadeOut;
  SetTexPal;
end;

procedure TGame.LoadLevel(LevNum: Integer);
  var f: File;
      T: TTerrain;
      M: MonsterRec;
      P: PMonster;
      i, j: Integer;
      Tmp: String;
begin
  Gold := 0;
  if BegQ <> nil then Dispose(BegQ, Done);

  for i := 1 to LEVEL_WIDTH div 20 do
  for j := 1 to LEVEL_HEIGHT div 14 do
    Level^[i, j] := 255;

  Str(LevNum, Tmp);
  if LevNum < 10 then Tmp := '0' + Tmp;

  {$I-}
  Assign(f, LEVEL_FILE + Tmp + '.dat');
  Reset(f, 1);
  {$I+}

  if IOResult <> 0 then
    Error(ERROR_2);

  BlockRead(f, LevelInfo, SizeOf(Info));

  for i := 1 to LevelInfo.NumTerrain do begin
    BlockRead(f, T, SizeOf(TTerrain));
    if T.n = MONEY_SPRITE then Inc(Gold);
    Level^[T.x div 20 + 1, T.y div 14 + 1] := T.n;
  end;

  for i := 1 to LevelInfo.NumMonsters do begin
    BlockRead(f, M, SizeOf(M));
    P := New(PMonster, Init(M.x, M.y, M.n, 1));
    P^.Next := nil;
    if BegQ = nil then BegQ := P
      else EndQ^.Next := P;
    EndQ := P;
  end;

  for i := 1 to LevelInfo.NumPlats do begin
  end;

  BlockRead(f, T, SizeOf(TTerrain));
  if T.n = 255 then begin
    SpawnX := T.x;
    SpawnY := T.y;
    Mario^.PutInLevel(T.x, T.y);
  end;

  Sizes;

  Close(f);

  ShowLevelNum;
end;


procedure TGame.LoadData;
  var f: File;
begin
  {$I-}
  Assign(f, DATA_FILE);
  Reset(f, 1);
  {$I+}

  if IOResult <> 0 then
    Error(ERROR_1);

  Seek(f, 54);
  BlockRead(f, Pal, 1024);
  GetMem(Data, DATA_SIZE);
  BlockRead(f, Data^, DATA_SIZE);

  Close(f);
end;

procedure TGame.Intro;
  var i, j: Integer;
      C, Count: Byte;
begin
  SetTexPal;
  for i := 202 to 218 do begin
    C := (i - 202) * 15;
    SetRGBPal(i, C, C, 255);
    C := 255 - (i - 202) * 15;
    SetRGBPal(i + 16, C, C, 255);
  end;
  Texture(0, 0, 0);
  OutString(71, 91, 'MARIO - by MATI Inc.', 0);
  OutString(70, 90, 'MARIO - by MATI Inc.', 250);
  PutImage(140, 60, 20, 14, 0, 76 * 14, 20, Data);
  for i := 1 to 10 do
  for j := 1 to 6 do
    if INTRO_DATA[j, i] <> 0 then
      PutImage(i * 20 + 40, j * 14 + 105, 20, 14, 0, INTRO_DATA[j, i] * 14, 20, Data);
  CopyToScreen(Screen);
  FadeIn;
  Count := 0;
  repeat
    Inc(Count);
    VertRet;
  until (KeyPressed) or (Count = 255);
  if KeyPressed then ReadKey;
  FadeOut;
end;

procedure TGame.DrawInfo;
  var S, S1: String;
begin
  Str(Mario^.Lives, S);
  S := 'Lives ' + S;
  OutString(251, 182, S, 0);
  OutString(250, 181, S, 250);
  PutImage(10, 180, 20, 14, 0, MONEY_SPRITE * 14, 20, Data);
  Str(Mario^.Gold, S);
  Str(Gold, S1);
  OutString(36, 182, S + '/' + S1, 0);
  OutString(35, 181, S + '/' + S1, 250);
end;

procedure TGame.RefreshScreen;
  var i, j: Integer;
begin
  if not UNREGISTERED then
    Texture(Screen_X, Screen_Y, 2)
  else
    Cls(0);

  for i := 4 to 6 do
    Dec(Speeds[i]);

  for i := Screen_X div 20 to (Screen_X div 20) + 17 do
  for j := Screen_Y div 14 to (Screen_Y div 14) + 16 do begin
    if Level^[i + 1, j + 1] <> 255 then
      PutImage(i * 20 - Screen_X, j * 14 - Screen_Y, 20, 14, 0, Level^[i + 1, j + 1] * 14, 20, Data);
    if Speeds[4] = 0 then begin
      if Level^[i + 1, j + 1] = BEG_FIRE then
        Level^[i + 1, j + 1] := BEG_FIRE + 2
      else
        if Level^[i + 1, j + 1] = BEG_FIRE + 2 then Level^[i + 1, j + 1] := BEG_FIRE;
    end;
    if Speeds[5] = 0 then begin
      if (Level^[i + 1, j + 1] > 7) and (Level^[i + 1, j + 1] < 15) then
        Level^[i + 1, j + 1] := Level^[i + 1, j + 1] + 7
      else
        if (Level^[i + 1, j + 1] > 14) and (Level^[i + 1, j + 1] < 22) then
          Level^[i + 1, j + 1] := Level^[i + 1, j + 1] - 7;
    end;
    if Speeds[6] = 0 then begin
      if (Level^[i + 1, j + 1] > WIZARD_BEG - 1) and (Level^[i + 1, j + 1] < WIZARD_BEG + 4) then begin
        Inc(Level^[i + 1, j + 1]);
        if Level^[i + 1, j + 1] = WIZARD_BEG + 4 then Level^[i + 1, j + 1] := 255;
      end;
    end;
  end;

  if Speeds[4] = 0 then begin
    Speeds[4] := SPEED_TABLE[4];
  end;
  if Speeds[5] = 0 then begin
    Speeds[5] := SPEED_TABLE[5];
  end;
  if Speeds[6] = 0 then begin
    Speeds[6] := SPEED_TABLE[6];
  end;

  if BegQ <> nil then BegQ^.Draw(Screen_X, Screen_Y);
  Mario^.Draw(Screen_X, Screen_Y);
  if Tab or AllowTab then DrawInfo;
  if UNREGISTERED then begin
    OutString(11, 11, MESSAGE, 0);
    OutString(10, 10, MESSAGE, RED_COLOR);
  end;

  VertRet;
  CopyToScreen(Screen);

  for i := 1 to 3 do
    Dec(Speeds[i]);

  if Speeds[1] = 0 then begin
    Speeds[1] := SPEED_TABLE[1];
    ScrollPal(12, 2);
  end;
  if Speeds[2] = 0 then begin
    Speeds[2] := SPEED_TABLE[2];
    ScrollPal(158, 1);
  end;
  if Speeds[3] = 0 then begin
    Speeds[3] := SPEED_TABLE[3];
    ScrollPal(7, 4);
  end;

end;

procedure TGame.GameOver;
begin
  DrawBack;
  OutString(121, 91, 'Game Over', 0);
  OutString(120, 90, 'Game Over ', 250);
  PutImage(150, 60, 20, 14, 0, 76 * 14, 20, Data);
  CopyToScreen(Screen);
  FadeIn;
  ReadKey;
  FadeOut;
end;

procedure TGame.Sizes;
  var x, y: Integer;
begin
  x := Mario^.MarioX;
  y := Mario^.MarioY;
  if x - Screen_X < 100 then Screen_X := x - 100;
  if (Screen_X < 0) then Screen_X := 0;
  if x - Screen_X > 200 then Screen_X := x - 200;
  if (Screen_X > LEVEL_WIDTH - 320) then Screen_X := LEVEL_WIDTH - 320;
  if y - Screen_Y < 80 then Screen_Y := y - 80;
  if (Screen_Y < 0) then Screen_Y := 0;
  if y - Screen_Y > 120 then Screen_Y := y - 120;
  if (Screen_Y > LEVEL_HEIGHT - 200) then Screen_Y := LEVEL_HEIGHT - 200;
end;

procedure TGame.run;
  var Key: Byte;
      Count: Integer;
begin
  RefreshScreen;
  FadeIn;
  Count := DEMO_TIME;

  repeat
    asm
      cli
      in al, 60h
      mov Key, al
    end;

    if Key = 15 then Tab := True;
    if Key = 143 then Tab := False;

    if BegQ <> nil then BegQ^.HandleEvent;
    if Demo then begin
      Dec(Count);
      if Count = 0 then begin
        FadeOut;
        Intro;
        SetTexPal;
        LoadLevel(CurrentLevel);
        RefreshScreen;
        FadeIn;
        Count := DEMO_TIME;
      end;
    end
    else Mario^.HandleEvent(Key, nil);

    if (Key = 88) and FindParam(PASSWORD_2) then begin
      WinGame;
      Exit;
    end;

    if (Mario^.Gold = Gold) or ((Key = 68) and (FindParam(PASSWORD_1))) then begin
      Inc(CurrentLevel);
      if CurrentLevel = MAX_LEVELS then begin
        WinGame;
        Exit;
      end;
      LoadLevel(CurrentLevel);
      Sizes;
      RefreshScreen;
      FadeIn;
    end;

    if Mario^.DeadFlag and not Demo then begin
      FadeOut;
      if Mario^.Lives >= 0 then begin
        LoadLevel(CurrentLevel);
        RefreshScreen;
      end
      else begin
        GameOver;
        Exit;
      end;
      FadeIn;
    end;

    Sizes;
    RefreshScreen;

    asm
      sti
      mov ax, 0C00h
      int 21h
    end;
  until Key = 1;

  FadeOut;
end;

procedure DrawWinGame(Offs: Integer);
  var i, j: Integer;
begin
  for i := 1 to 10 do
  for j := 1 to 6 do
    if INTRO_DATA[j, i] <> 0 then
      PutImage(i * 20 + 40, j * 14 + 105, 20, 14, 0, INTRO_DATA[j, i] * 14, 20, Data);
  for i := 1 to 16 do begin
    OutString(11, Offs + (i - 1) * 14 + 1, CONGRATULATIONS[i], 0);
    OutString(10, Offs + (i - 1) * 14, CONGRATULATIONS[i], 250);
  end;
  Flip__H(20, 14, 76 * 20 * 14, Data);
  PutImage(20, 10, 20, 14, 0, 76 * 14, 20, Data);
  Flip__H(20, 14, 76 * 20 * 14, Data);
  PutImage(280, 10, 20, 14, 0, 76 * 14, 20, Data);
end;

procedure TGame.WinGame;
  var Offs: Integer;
begin
  Texture(0, 0, 1);
  DrawWinGame(Offs);
  CopyToScreen(Screen);
  FadeIn;
  Offs := 200;

  repeat
    VertRet;
    Texture(0, 0, 1);
    DrawWinGame(Offs);
    Dec(Offs);
    VertRet;
    CopyToScreen(Screen);
  until Offs < - 200;

  Delay(1000);
  ReadKey;
  FadeOut;
end;

end.